@@ -3,7 +3,7 @@ from datetime import datetime
 
 from django import forms
 from django.conf import settings
-from django.conf.urls.defaults import patterns, url
+from django.conf.urls import patterns, url
 from django.contrib import admin
 from django.core.management import get_commands
 from django.core.urlresolvers import reverse, NoReverseMatch
@@ -79,9 +79,8 @@ class JobAdmin(admin.ModelAdmin):
         'last_run_with_link',
         'get_timeuntil',
         'get_frequency',
+        'job_type',
         'enabled',
-        #'force_run',
-        #'check_is_running',
         'check_is_complete',
         'is_fresh',
         #'is_monitor',
@@ -92,8 +91,17 @@ class JobAdmin(admin.ModelAdmin):
         'stop_button',
         'view_logs_button',
     )
+    
+    def job_type(self, obj=''):
+        if not obj:
+            return ''
+        if obj.is_monitor:
+            return 'monitor'
+        return 'job'
+    job_type.short_description = 'type'
+    
     readonly_fields = (
-        'check_is_running',
+        #'check_is_running',
         'check_is_complete',
         'view_logs_button',
         'last_run_successful',
@@ -105,6 +113,9 @@ class JobAdmin(admin.ModelAdmin):
         'total_parts_complete',
         'progress_percent_str',
         'estimated_completion_datetime_str',
+        'monitor_records',
+        'current_hostname',
+        'job_type',
     )
     list_display_links = ('name', )
     list_filter = (
@@ -122,14 +133,16 @@ class JobAdmin(admin.ModelAdmin):
                 'command',
                 'args',
                 'hostname',
+                'current_hostname',
                 'enabled',
-                'check_is_running',
+                #'check_is_running',
+                'is_running',
                 'force_run',
                 'force_stop',
                 'view_logs_button',
                 'last_run_successful',
-                'last_heartbeat',
                 'is_fresh',
+                'last_heartbeat',
                 'last_run_start_timestamp',
                 'last_run',
                 'total_parts',
@@ -141,6 +154,7 @@ class JobAdmin(admin.ModelAdmin):
                 'monitor_url',
                 'monitor_error_template',
                 'monitor_description',
+                'monitor_records',
             )
         }),
         ('E-mail subscriptions', {
@@ -156,21 +170,32 @@ class JobAdmin(admin.ModelAdmin):
             'fields': ('frequency', 'next_run', 'params',)
         }),
     )
-    search_fields = ('name', )
+    search_fields = (
+        'name',
+        'hostname',
+        'current_hostname',
+    )
     
     inlines = (
        JobDependencyInline,
     )
     
+    def get_readonly_fields(self, request, obj=None):
+        fields = list(self.readonly_fields)
+        # Allow manual clearing of is_running if the cron job has become stuck.
+        if obj and obj.is_fresh():
+            fields.append('is_running')
+        return fields
+    
     def last_run_with_link(self, obj=None):
         if not obj or not obj.id:
             return ''
         format = get_format('DATETIME_FORMAT')
         value = None
-        if obj.last_run is not None:
-            value = timezone.localtime(obj.last_run)
-            value = capfirst(dateformat.format(value, format))
         try:
+            if obj.last_run is not None:
+                value = timezone.localtime(obj.last_run)
+                value = capfirst(dateformat.format(value, format))
             log_id = obj.log_set.latest('run_start_datetime').id
             try:
                 # Old way
@@ -193,7 +218,7 @@ class JobAdmin(admin.ModelAdmin):
     check_is_complete.boolean = True
     
     def get_timeuntil(self, obj=None):
-        if not obj or not obj.id:
+        if not obj or not obj.id or not obj.next_run:
             return ''
         format = get_format('DATETIME_FORMAT')
         value = capfirst(dateformat.format(timezone.localtime(obj.next_run), format))
@@ -497,6 +522,7 @@ class MonitorAdmin(admin.ModelAdmin):
     list_display = (
         'name_str',
         'status',
+        'monitor_records',
         'get_timeuntil',
         'enabled',
         'action_buttons',
@@ -506,6 +532,7 @@ class MonitorAdmin(admin.ModelAdmin):
     )
     readonly_fields = (
         'name_str',
+        'monitor_records',
         'action_buttons',
         'status',
         'get_timeuntil',
