@@ -21,7 +21,6 @@ import settings as _settings
 
 import django
 from django.conf import settings
-from django.contrib.auth.models import User
 from django.contrib.sites.models import Site
 from django.core.mail import send_mail
 from django.core.management import call_command
@@ -95,20 +94,24 @@ class JobManager(models.Manager):
         # Note, select_for_update() may not be supported, such as with MySQL.
         # Those backends will need to use an explicit backend-specific locking
         # mechanism.
-        q = self.select_for_update(nowait=False)
+        if settings.CHRONIKER_SELECT_FOR_UPDATE:
+            q = self.select_for_update(nowait=False)
+        else:
+            q = self.all()
         q = q.filter(Q(next_run__lte=timezone.now()) | Q(force_run=True))
         q = q.filter(
             Q(hostname__isnull=True) | \
             Q(hostname='') | \
             Q(hostname=socket.gethostname()))
         q = q.filter(enabled=True)
+        q = q.filter(is_running=False)
         if job is not None:
             if isinstance(job, int):
                 job = job.id
             q = q.filter(id=job.id)
         return q
 
-    def due_with_met_dependencies(self):
+    def due_with_met_dependencies(self, jobs=[]):
         """
         Iterates over the results of due(), ignoring jobs
         that are dependent on another job that is also due.
@@ -124,18 +127,49 @@ class JobManager(models.Manager):
 #            return 0
 #        
 #        return sorted(self.due(), cmp=cmp_deps)
-
+        
+        # Fixes the "Lost connection to MySQL server during query" error when
+        # called from cron command?
+        connection.close()
+        
+        skipped_job_ids = set()
         for job in self.due():
+            if jobs and job.id not in jobs:
+                print 'Skipping job %i (%s) because jobs are limited to %s.' % (job.id, job, ', '.join(map(str, jobs)))
+                skipped_job_ids.add(job.id)
+                continue
+            
             deps = job.dependencies.all()
             valid = True
+            
             if job.check_is_running():
+                print 'Skipping job %i (%s) which is already running.' % (job.id, job)
+                #skipped_job_ids.add(job.id)
                 continue
+            
+            failed_dep = None
             for dep in deps:
-                if dep.wait_for_completion and dep.dependee.is_due():
+                if dep.dependee.id in skipped_job_ids:
+                    continue
+                #elif dep.wait_for_completion and dep.dependee.is_due():
+                elif not dep.criteria_met():
                     valid = False
+                    failed_dep = dep
                     break
-            if valid and job.dependencies_met():
-                yield job
+                
+            if not valid:
+                print 'Skipping job %i (%s) which is dependent on a due job %i (%s).' \
+                    % (job.id, job, failed_dep.dependee.id, failed_dep.dependee)
+                skipped_job_ids.add(job.id)
+                continue
+            
+            #TODO:remove? redundant?
+            if not job.dependencies_met():
+                print 'Skipping job %i (%s) which has unmet dependencies.' % (job.id, job)
+                skipped_job_ids.add(job.id)
+                continue
+            
+            yield job
 
     def stale(self):
         q = self.filter(is_running=True)
@@ -143,6 +177,9 @@ class JobManager(models.Manager):
             Q(last_heartbeat__isnull=True) | 
             Q(last_heartbeat__lt=timezone.now() - timedelta(minutes=5)))
         return q
+    
+    def all_running(self):
+        return self.filter(is_running=True)
 
 class JobHeartbeatThread(threading.Thread):
     """
@@ -223,9 +260,15 @@ class JobDependency(models.Model):
     Represents a scheduling dependency between two jobs.
     """
     
-    dependent = models.ForeignKey('Job', related_name='dependencies')
+    dependent = models.ForeignKey(
+        'Job',
+        related_name='dependencies',
+        help_text='The thing that cannot run until another job completes.')
     
-    dependee = models.ForeignKey('Job', related_name='dependents')
+    dependee = models.ForeignKey(
+        'Job',
+        related_name='dependents',
+        help_text='The thing that has other jobs waiting on it to complete.')
     
     wait_for_completion = models.BooleanField(
         default=True,
@@ -344,7 +387,8 @@ class Job(models.Model):
     
     is_running = models.BooleanField(
         default=False,
-        editable=False)
+        editable=True,
+    )
     
     last_run_successful = models.NullBooleanField(
         _('success'),
@@ -353,7 +397,7 @@ class Job(models.Model):
         editable=False)
     
     subscribers = models.ManyToManyField(
-        User,
+        settings.AUTH_USER_MODEL,
         related_name='subscribed_jobs',
         blank=True,
         limit_choices_to={'is_staff':True})
@@ -382,14 +426,22 @@ class Job(models.Model):
         help_text=_("If checked and running then this job will be stopped."))
     
     hostname = models.CharField(
-        max_length=255,
+        max_length=700,
         blank=True,
         null=True,
+        verbose_name='target hostname',
         help_text=_('If given, ensures the job is only run on the server ' + \
             'with the equivalent host name.<br/>Not setting any hostname ' + \
             'will cause the job to be run on the first server that ' + \
             'processes pending jobs.<br/> ' + \
-            'The current hostname is <b>%s</b>.') % socket.gethostname())
+            'e.g. The hostname of this server is <b>%s</b>.') % socket.gethostname())
+    
+    current_hostname = models.CharField(
+        max_length=700,
+        blank=True,
+        null=True,
+        editable=False,
+        help_text=_('The name of the host currently running the job.'))
     
     total_parts_complete = models.PositiveIntegerField(
         default=0,
@@ -424,6 +476,13 @@ class Job(models.Model):
         blank=True, null=True,
         help_text=_('An explanation of the monitor\'s purpose.'))
     
+    monitor_records = models.IntegerField(
+        blank=True,
+        null=True,
+        #verbose_name='records',
+        editable=False,
+        help_text=_('The number of records that needs attention.'))
+    
     maximum_log_entries = models.PositiveIntegerField(
         default=1000,
         help_text='The maximum number of most recent log entries to keep.' + \
@@ -439,7 +498,7 @@ class Job(models.Model):
     
     def __unicode__(self):
         if not self.enabled:
-            return _(u"%(name)s - disabled") % {'name': self.name}
+            return _(u"%(id)s - %(name)s - disabled") % {'name': self.name, 'id':self.id}
         return u"%s - %s" % (self.name, self.timeuntil)
     
     @property
@@ -523,13 +582,32 @@ class Job(models.Model):
             if not self.next_run or j.params != self.params:
                 logger.debug("Updating 'next_run")
                 next_run = self.next_run or timezone.now()
-                self.next_run = self.rrule.after(next_run)
+                
+                #TODO:why can't rrule consistently handle timezone-aware datetimes?!?!?!
+                tz = timezone.get_default_timezone()
+                try:
+                    self.next_run = self.rrule.after(timezone.make_aware(next_run, tz))
+                except TypeError, e:
+                    self.next_run = timezone.make_aware(
+                        self.rrule.after(timezone.make_naive(next_run, tz)),
+                        tz)
+        
+        #old = None
+        #if self.id:
+        #    old = Job.objects.get(id=self.id)
+        
+        if not self.is_running:
+            self.current_hostname = None
         
         super(Job, self).save(*args, **kwargs)
         
+        #if old:
+        #    print 'is_running_changed:', old.is_running != self.is_running, old.is_running, '->', self.is_running
+        
         # Delete expired logs.
         if self.maximum_log_entries:
-            for o in self.logs.all().order_by('-run_start_datetime')[self.maximum_log_entries:]:
+            log_q = self.logs.all().order_by('-run_start_datetime')
+            for o in log_q[self.maximum_log_entries:]:
                 o.delete()
 
     def dependencies_met(self):
@@ -727,6 +805,7 @@ class Job(models.Model):
         subprocess, which can be invoked by calling this ``Job``\'s ``run``
         method.
         """
+        print 'Handling run...'
         
         lock = threading.Lock()
         run_start_datetime = timezone.now()
@@ -755,6 +834,7 @@ class Job(models.Model):
                 job = Job.objects.get(id=self.id)
                 job.is_running = True
                 job.last_run_start_timestamp = timezone.now()
+                job.current_hostname = socket.gethostname()
                 job.total_parts = 0
                 job.total_parts_complete = 0
                 if heartbeat:
@@ -827,7 +907,6 @@ class Job(models.Model):
                 if job.last_run_successful and job.total_parts is not None:
                     job.total_parts_complete = job.total_parts
                 job.save()
-                #django.db.transaction.commit()
             except Exception, e:
                 # The command failed to run; log the exception
                 t = loader.get_template('chroniker/error_message.txt')
@@ -898,20 +977,27 @@ class Job(models.Model):
         Currently, it only supports `posix` systems.  On non-posix systems
         it returns the value of this job's ``is_running`` field.
         """
-        if self.is_running and self.lock_file:
+        if settings.CHRONIKER_CHECK_LOCK_FILE \
+        and self.is_running and self.lock_file:
             # The Job thinks that it is running, so lets actually check
+            # NOTE: This will screw up the record if separate hosts
+            # are processing and reading the jobs.
             if os.path.exists(self.lock_file):
                 # The lock file exists, but if the file hasn't been modified
                 # in less than LOCK_TIMEOUT seconds ago, we assume the process
-                # is dead
-                if (time.time() - os.stat(self.lock_file).st_mtime) <= settings.CHRONIKER_LOCK_TIMEOUT:
+                # is dead.
+                if (time.time() - os.stat(self.lock_file).st_mtime) \
+                <= settings.CHRONIKER_LOCK_TIMEOUT:
                     return True
             
             # This job isn't running; update it's info
             self.is_running = False
             self.lock_file = ""
             self.save()
-        return False
+            return False
+        else:
+            # We assume the database record is definitive.
+            return self.is_running
     check_is_running.short_description = "is running"
     check_is_running.boolean = True
     
