@@ -49,7 +49,24 @@ settings.CHRONIKER_PID_FN = getattr(
 
 # If true, uses a PID file to ensure the cron management command only runs
 # one at a time.
+# TODO: unnecessary? deprecate?
 settings.CHRONIKER_USE_PID = getattr(
     settings,
     'CHRONIKER_USE_PID',
+    False)
+
+settings.CHRONIKER_SELECT_FOR_UPDATE = getattr(
+    settings,
+    'CHRONIKER_SELECT_FOR_UPDATE',
     True)
+
+# Set this to false for use on multiple hosts, since
+# the lock file will only be accessible on a single host.
+# The database will be used as effective lock.
+# Recommend setting CHRONIKER_SELECT_FOR_UPDATE = True.
+# Only set this to true if only a single host will ever read and write
+# to the Job table.
+settings.CHRONIKER_CHECK_LOCK_FILE = getattr(
+    settings,
+    'CHRONIKER_CHECK_LOCK_FILE',
+    False)
